<?php

namespace Drupal\gatekeepers_age\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\gatekeepers_age\Constants\AgeGateDivisions;
use Drupal\gatekeepers_age\Constants\AgeGateLabels;
use Drupal\gatekeepers_age\Constants\AgeGateSettings;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class AgeGateController.
 *
 * @noinspection PhpUnused
 */
class AgeGateController extends ControllerBase {

  /**
   * Get the Age Gate configuration values.
   *
   * @noinspection PhpUnused
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The retrieved Age Gate configuration values.
   */
  public function getAge() : JsonResponse {
    $data = [];

    // Append necessary data.
    $this->appendLabels($data);
    $this->appendDivisions($data);

    return new JsonResponse($data);
  }

  /**
   * Append the labels data to the provided data array.
   *
   * @param array $data
   *   The provided data array.
   */
  private function appendLabels(array &$data) : void {
    $labels = $this->config(AgeGateSettings::CONFIG_ID)->get(AgeGateLabels::ID);
    $data[AgeGateLabels::ID] = $labels;
  }

  /**
   * Append the age data to the provided data array.
   *
   * @param array $data
   *   The provided data array.
   */
  private function appendDivisions(array &$data) : void {
    $divisions = $this->config(AgeGateSettings::CONFIG_ID)->get(AgeGateDivisions::ID);
    $this->fillBlanks($divisions);
    $data[AgeGateDivisions::ID] = $divisions;
  }

  /**
   * Fill blank age values with its closest non-blank parent value.
   *
   * The first iteration should always contain the topmost non-blank default
   * value.
   *
   * @param array $values
   *   The supplied values.
   * @param int $parent
   *   Optional non-blank parent value.
   */
  private function fillBlanks(array &$values, int $parent = 0) : void {
    foreach ($values as &$properties) {
      // Fill blank age value.
      if (empty($properties[AgeGateDivisions::PROPERTY_AGE])) {
        $properties[AgeGateDivisions::PROPERTY_AGE] = (string) $parent;
      }

      // Recurse into sub-divisions.
      if (!empty($properties[AgeGateDivisions::PROPERTY_DIVISIONS])) {
        $this->fillBlanks($properties[AgeGateDivisions::PROPERTY_DIVISIONS], $properties[AgeGateDivisions::PROPERTY_AGE]);
      }
    }
  }

}
